/*
	FILE		: peregrine_in_vivo.h
	DESCRIPTION	: Peregrine in vivo API providing:
					1: Peregrine Test Probes for automated in vivo testing of (embedded) software.
					2: Interprocess communication between the Peregrine program (Tester) and the program under test (Testee).
	COPYRIGHT	: (c) 2017-2023 Fluxica Computer Engineering
	CONTACT		: info@fluxica.com
*/

/* 
	A test is set up in four steps:

    1. Include this header file.
    2. Add two Peregrine code delimiters around the code segment to be tested.
    3. Replace (selected) variables with matching Peregrine Test Probe types in the marked out code segment.
    4. Compile the program source code to create an executable file as usual and additionally link against the Peregrine library.

	Invoking the Peregrine program and then running the executable file will automatically test the code segment. For more 
	information, please see the Fluxica Peregrine Reference Manual.
*/


/*
	The include macros below can be removed if the required header files are already included in Testee.
*/

#ifndef INTTYPES_H
	#define INTTYPES_H
	#include <inttypes.h>
#endif

#ifndef STDBOOL_H
	#define STDBOOL_H
	#include <stdbool.h>
#endif

/* 
	MACRO DEFINITIONS PEREGRINE IN VIVO CODE DELIMITERS 
*/

#define PEREGRINE_RED_CLIP		peregrine_constructor(); while(peregrine_flag()) {
#define PEREGRINE_BLACK_CLIP	} peregrine_destructor();

/* 
	MACRO DEFINITION PEREGRINE IN VIVO FAILURE DETECTED
*/

#define PEREGRINE_ORACLE	peregrine_oracle()

/*	
	MACRO DEFINITIONS PEREGRINE IN VIVO TEST PROBES 

	The semicolon after the Peregrine Test Probe (PTP) is provided by the calling
	program (Testee) if required, making both variable assignment and variable 
	passing possible for PTPs.

	The trailing number of a PTP defines the number of bits that are actually used, 
	the address space in memory is always at least 8 bits wide.

	A PTP is not similar or equivalent to a C data type. A PTP provides a value of 
	a given type that will have changed its value before it is read again. The next 
	value does not depend on previous values. The values have a uniform (pseudo-)random 
	distribution, they are in every practical sense without any bias.
*/

#define UINT64			peregrine_uint64()
#define INT64			peregrine_int64()	
#define UINT32			peregrine_uint32()
#define INT32			peregrine_int32()
#define UINT16			peregrine_uint16()
#define INT16			peregrine_int16()
#define UINT8			peregrine_uint8()
#define INT8			peregrine_int8()
#define UINT4			peregrine_uint4() 
#define INT4			peregrine_int4()
#define UINT2			peregrine_uint2()
#define INT2			peregrine_int2()
#define UINT1			peregrine_uint1()
#define BOOL1			peregrine_bool1()
#define CHAR8			peregrine_char8()
#define UCHAR8			peregrine_uchar8()
#define FLOAT32			peregrine_float32()
#define DOUBLE64		peregrine_double64()

#ifdef __cplusplus
extern "C" {
#endif

/* 
	FUNCTION DECLARATIONS PEREGRINE IN VIVO IPC
*/

void peregrine_constructor();
bool peregrine_flag();
void peregrine_destructor();
void peregrine_oracle();

/*
	FUNCTION DECLARATIONS PEREGRINE IN VIVO TEST PROBES
*/

uint64_t		peregrine_uint64();
int64_t			peregrine_int64();
uint32_t		peregrine_uint32();
int32_t 		peregrine_int32();
uint16_t 		peregrine_uint16();
int16_t			peregrine_int16();
uint8_t	 		peregrine_uint8();
int8_t 			peregrine_int8();
uint8_t 		peregrine_uint4();
int8_t 			peregrine_int4();
uint8_t 		peregrine_uint2();
int8_t 			peregrine_int2();
uint8_t 		peregrine_uint1();
bool 			peregrine_bool1();
char			peregrine_char8();
unsigned char	peregrine_uchar8();
float 			peregrine_float32();
double 			peregrine_double64();

#ifdef __cplusplus
}
#endif
