#!/bin/bash

#	FILE		: cppjava.sh
#	DESCRIPTION	: Bash command line interface script to preprocess and compile Java source code.
#	COPYRIGHT	: (c) 2017-2023 Fluxica Computer Engineering
#	CONTACT		: info@fluxica.com

#	SYNOPSIS	: ./cppjava.sh <JavaSource.java>

<< NOTE
	The java compiler requires that the Java file name matches the name of the public class in the file.
	This requirement does not apply if the Java file only contains local classes.
	
	An alternative to the script below is to work in a Java file with another extension than ".java".
	For example, if the working Java file has a ".jv" extension:
	Preprocessing: cpp -P JavaSource.jv JavaSource.java
	Compiling: javac JavaSource.java
NOTE

CPP_CMD="cpp -P"
JAVAC_CMD="javac"

echo -e "Bash shell script to preprocess and compile Java source code.\n"

if [[ -z $1 ]]; then
	echo -e "- Script argument required (*.java).\n"
	exit
fi

if [[ $1 != *.java ]]; then
	echo -e "- $1 does not have .java extension.\n"	
	exit
else 
	JAVA_FILE=$1
	JAVA_FILE_ORG="$JAVA_FILE"_org
	JAVA_FILE_TMP="$JAVA_FILE"_tmp
	CLASS_FILE=${JAVA_FILE/%".java"}.class
fi

if [[ -f $JAVA_FILE_ORG ]]; then 
	rm $JAVA_FILE_ORG
	if [[ -f $JAVA_FILE_ORG ]]; then 
		echo -e "- $JAVA_FILE_ORG file not removed.\n"
		exit
	fi
fi

if [[ -f $JAVA_FILE_TMP ]]; then 
	rm $JAVA_FILE_TMP
	if [[ -f $JAVA_FILE_TMP ]]; then 
		echo -e "- $JAVA_FILE_TMP file not removed.\n"
		exit
	fi
fi

echo -e "The following operations will be performed:\n"
echo -e "- $JAVA_FILE will be copied to $JAVA_FILE_ORG"
echo -e "- $JAVA_FILE will be changed to include macro expansions"
echo -e "- $JAVA_FILE will be compiled to create $CLASS_FILE"
echo -e "- $JAVA_FILE_ORG will be moved to $JAVA_FILE\n"

read -p "Continue [y/n]? " CONFIRM

echo ""

if [[ $CONFIRM == 'y' ]]; then
	if [[ -f $JAVA_FILE ]]; then 
		cp $JAVA_FILE $JAVA_FILE_ORG
		
		if [[ ! -f $JAVA_FILE_ORG ]]; then 
			echo -e "- $JAVA_FILE file not copied.\n"
			exit
		fi
	
		$CPP_CMD $JAVA_FILE_ORG $JAVA_FILE_TMP
		if [[ $? -eq 1 ]]; then
			echo -e "\n- Preprocessing did not succeed.\n"
			rm $JAVA_FILE_ORG
			exit
		fi
	
		if [[ ! -f $JAVA_FILE_TMP ]]; then 
			echo -e "- $JAVA_FILE_TMP file not created.\n"
			rm $JAVA_FILE_ORG
			exit
		else
			mv $JAVA_FILE_TMP $JAVA_FILE
			$JAVAC_CMD $JAVA_FILE
			if [[ $? -eq 1 ]]; then 
				echo -e "\n- Compilation did not succeed.\n"
				mv $JAVA_FILE_ORG $JAVA_FILE
				exit
			fi
			mv $JAVA_FILE_ORG $JAVA_FILE
		fi
	
		if [[ ! -f $CLASS_FILE ]]; then 
			echo -e "- $CLASS_FILE file not created.\n"
			exit
		else
			echo -e "- $CLASS_FILE file created.\n"
		fi
	else 
		echo -e "- $JAVA_FILE file does not exist.\n"
	fi
else 
	echo -e "- No operation performed.\n"
fi

